<?php

namespace Tests\Feature;

use App\Models\Server;
use App\Models\User;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Tests\TestCase;

class ConnectTest extends TestCase
{
    use RefreshDatabase;

    public function test_full_good() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
            'public' => true
        ]);

        $user_token = Auth::guard('client')->login($user);
        $server_token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $user_token
        ])->post('/api/server/' . $server->id . '/connect', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(204);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $server_token
        ])->post('/api/user/' . $user->id . '/accept', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(204);
    }

    public function test_connect_already_connected() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
            'public' => true
        ]);

        $user_token = Auth::guard('client')->login($user);
        $server_token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $user_token
        ])->post('/api/server/' . $server->id . '/connect', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(204);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $server_token
        ])->post('/api/user/' . $user->id . '/accept', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(204);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $user_token
        ])->post('/api/server/' . $server->id . '/connect', [
            'token' => '0705183cee5b5e7bbe0056ca9637011e095abde3b8b4968e0d03939c78700b8a'
        ]);
        $response->assertStatus(403);
    }

    public function test_accept_not_connecting() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
            'public' => true
        ]);

        $user_token = Auth::guard('client')->login($user);
        $server_token = Auth::guard('server')->login($server);
        
        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $server_token
        ])->post('/api/user/' . $user->id . '/accept', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(403);
    }

    public function test_accept_wrong_token() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
            'public' => true
        ]);

        $user_token = Auth::guard('client')->login($user);
        $server_token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $user_token
        ])->post('/api/server/' . $server->id . '/connect', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(204);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $server_token
        ])->post('/api/user/' . $user->id . '/accept', [
            'token' => '0705183cee5b5e7bbe0056ca9637011e095abde3b8b4968e0d03939c78700b8a'
        ]);
        $response->assertStatus(403);
    }

    public function test_accept_wrong_server() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server2 = Server::create([
            'login' => 'test2',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
            'public' => true
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
            'public' => true
        ]);

        $user_token = Auth::guard('client')->login($user);
        $server_token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $user_token
        ])->post('/api/server/' . $server2->id . '/connect', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(204);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $server_token
        ])->post('/api/user/' . $user->id . '/accept', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(403);
    }
}
