<?php

namespace Tests\Feature;

use App\Models\Server;
use App\Models\User;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Tests\TestCase;

class GameCreationTest extends TestCase
{
    use RefreshDatabase;

    public function test_good()
    {
        $user1 = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $user2 = User::create([
            'username' => 'test2',
            'email' => 'test2@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user1->id,
            'public' => true
        ]);

        $user1->server_id = $server->id;
        $user1->server_state = 'connected';
        $user1->save();

        $user2->server_id = $server->id;
        $user2->server_state = 'connected';
        $user2->save();

        $token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->postJson('/api/game', [
            "users" => [[
                "id" => $user1->id
            ], [
                "id" => $user2->id
            ]]
        ]);

        $response->assertStatus(201);
        $this->assertDatabaseCount("games", 1);
        $this->assertDatabaseCount("users_games", 2);
    }

    public function test_user_not_connected()
    {
        $user1 = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $user2 = User::create([
            'username' => 'test2',
            'email' => 'test2@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user1->id,
            'public' => true
        ]);

        $user1->server_id = $server->id;
        $user1->server_state = 'connected';
        $user1->save();

        $token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->postJson('/api/game', [
            "users" => [[
                "id" => $user1->id
            ], [
                "id" => $user2->id
            ]]
        ]);

        $response->assertStatus(403);
        $this->assertDatabaseCount("games", 0);
        $this->assertDatabaseCount("users_games", 0);
    }

    public function test_not_enough_users()
    {
        $user1 = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user1->id,
            'public' => true
        ]);

        $user1->server_id = $server->id;
        $user1->server_state = 'connected';
        $user1->save();

        $token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->postJson('/api/game', [
            "users" => [[
                "id" => $user1->id
            ]]
        ]);

        $response->assertStatus(422);
        $this->assertDatabaseCount("games", 0);
        $this->assertDatabaseCount("users_games", 0);
    }

    public function test_not_same_users()
    {
        $user1 = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user1->id,
            'public' => true
        ]);

        $user1->server_id = $server->id;
        $user1->server_state = 'connected';
        $user1->save();

        $token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->postJson('/api/game', [
            "users" => [[
                "id" => $user1->id
            ], [
                "id" => $user1->id
            ]]
        ]);

        $response->assertStatus(422);
        $this->assertDatabaseCount("games", 0);
        $this->assertDatabaseCount("users_games", 0);
    }

    public function test_user_connected_to_other_server()
    {
        $user1 = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $user2 = User::create([
            'username' => 'test2',
            'email' => 'test2@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user1->id,
            'public' => true
        ]);

        $server2 = Server::create([
            'login' => 'test2',
            'password' => Hash::make('test'),
            'owner_id' => $user1->id,
            'public' => true
        ]);

        $user1->server_id = $server->id;
        $user1->server_state = 'connected';
        $user1->save();

        $user2->server_id = $server2->id;
        $user2->server_state = 'connected';
        $user2->save();

        $token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->postJson('/api/game', [
            "users" => [[
                "id" => $user1->id
            ], [
                "id" => $user2->id
            ]]
        ]);

        $response->assertStatus(403);
        $this->assertDatabaseCount("games", 0);
        $this->assertDatabaseCount("users_games", 0);
    }

}
